@@ -1,4 +1,4 @@
-# $Id: references.py 7624 2013-03-07 14:10:26Z milde $
+# $Id: references.py 8387 2019-09-06 13:16:34Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -475,17 +475,17 @@ class Footnotes(Transform):
           # Entries 1-4 and 6 below are from section 12.51 of
           # The Chicago Manual of Style, 14th edition.
           '*',                          # asterisk/star
-          '\u2020',                    # dagger &dagger;
-          '\u2021',                    # double dagger &Dagger;
-          '\u00A7',                    # section mark &sect;
-          '\u00B6',                    # paragraph mark (pilcrow) &para;
+          u'\u2020',                    # dagger &dagger;
+          u'\u2021',                    # double dagger &Dagger;
+          u'\u00A7',                    # section mark &sect;
+          u'\u00B6',                    # paragraph mark (pilcrow) &para;
                                         # (parallels ['||'] in CMoS)
           '#',                          # number sign
           # The entries below were chosen arbitrarily.
-          '\u2660',                    # spade suit &spades;
-          '\u2665',                    # heart suit &hearts;
-          '\u2666',                    # diamond suit &diams;
-          '\u2663',                    # club suit &clubs;
+          u'\u2660',                    # spade suit &spades;
+          u'\u2665',                    # heart suit &hearts;
+          u'\u2666',                    # diamond suit &diams;
+          u'\u2663',                    # club suit &clubs;
           ]
 
     def apply(self):
@@ -663,7 +663,7 @@ class Substitutions(Transform):
     def apply(self):
         defs = self.document.substitution_defs
         normed = self.document.substitution_names
-        subreflist = self.document.traverse(nodes.substitution_reference)
+        subreflist = list(self.document.traverse(nodes.substitution_reference))
         nested = {}
         for ref in subreflist:
             refname = ref['refname']
@@ -692,14 +692,12 @@ class Substitutions(Transform):
                      or 'trim' in subdef.attributes):
                     if index > 0 and isinstance(parent[index - 1],
                                                 nodes.Text):
-                        parent.replace(parent[index - 1],
-                                       parent[index - 1].rstrip())
+                        parent[index - 1] = parent[index - 1].rstrip()
                 if  ('rtrim' in subdef.attributes
                      or 'trim' in subdef.attributes):
                     if  (len(parent) > index + 1
                          and isinstance(parent[index + 1], nodes.Text)):
-                        parent.replace(parent[index + 1],
-                                       parent[index + 1].lstrip())
+                        parent[index + 1] = parent[index + 1].lstrip()
                 subdef_copy = subdef.deepcopy()
                 try:
                     # Take care of nested substitution references:
@@ -710,6 +708,7 @@ class Substitutions(Transform):
                             raise CircularSubstitutionDefinitionError
                         else:
                             nested[nested_name].append(key)
+                            nested_ref['ref-origin'] = ref
                             subreflist.append(nested_ref)
                 except CircularSubstitutionDefinitionError:
                     parent = ref.parent
@@ -721,9 +720,13 @@ class Substitutions(Transform):
                             line=parent.line, base_node=parent)
                         parent.replace_self(msg)
                     else:
+                        # find original ref substitution which cased this error
+                        ref_origin = ref
+                        while ref_origin.hasattr('ref-origin'):
+                            ref_origin = ref_origin['ref-origin']
                         msg = self.document.reporter.error(
-                            'Circular substitution definition referenced: "%s".'
-                            % refname, base_node=ref)
+                            'Circular substitution definition referenced: '
+                            '"%s".' % refname, base_node=ref_origin)
                         msgid = self.document.set_id(msg)
                         prb = nodes.problematic(
                             ref.rawsource, ref.rawsource, refid=msgid)
@@ -863,7 +866,7 @@ class DanglingReferences(Transform):
 
 
 class DanglingReferencesVisitor(nodes.SparseNodeVisitor):
-    
+
     def __init__(self, document, unknown_reference_resolvers):
         nodes.SparseNodeVisitor.__init__(self, document)
         self.document = document
@@ -893,7 +896,10 @@ class DanglingReferencesVisitor(nodes.SparseNodeVisitor):
                 msgid = self.document.set_id(msg)
                 prb = nodes.problematic(
                       node.rawsource, node.rawsource, refid=msgid)
-                prbid = self.document.set_id(prb)
+                try:
+                    prbid = node['ids'][0]
+                except IndexError:
+                    prbid = self.document.set_id(prb)
                 msg.add_backref(prbid)
                 node.replace_self(prb)
         else:
